/*
	ProWesS reader - a HTML file browser

	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System

	Application window handling routines
		create the application window
		display the contents of the application window
		make sure the scrollbars are up to date

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 17 November 1995
	11.05.1996 added keypresses for scrolling
*/

#include "str_h"

#include "reader_h"
#include "HyperText_h"

/* configurable aspects of menu */

extern char PaperWhite;

#define MARGIN short2pt(5)

static Error scale(PWObject object, CanvasInfo *info)
{
	Global *g;
	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	g->canvaswidth=info->xsiz-2*MARGIN;
	g->Xpt2pix=info->Xpt2pix;
	g->Ypt2pix=info->Ypt2pix;
	return ERR_OK;
}

static Error draw(PWObject object, CanvasInfo *info)
{
	Global *g;
	TextLine *line;
	LineElement *el;
	pt xpos=MARGIN-info->xorg, ypos=MARGIN-info->yorg;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	if (!g->text) return ERR_OK;

	/* clear page */
	PFDisplayMode(info->gstate,PF_DISPLAYMODE_UPDATE);
	PFPaperColourGray(info->gstate, PaperWhite ? 0 : pt_hundred);
	PFPageClear(info->gstate);
	PFColourGray(info->gstate, PaperWhite ? pt_hundred : 0);
	PFPathMethod(info->gstate,PF_PATH_STROKED);

	/* make the y position correct cfr scrolling */
	for ( line=g->text->lines.next ;
		  (line!=&g->text->lines) && ypos+line->height<0 ;
		  ypos+=line->height, line=line->next);

	g->ypos=ypos+info->yorg;
	g->line=line;
	g->yinline=-ypos;

	if (line!=&g->text->lines)
	{
		/* select current font and size */
		PFFontLoad(info->gstate, line->styleline->font);
		PFFontScale(info->gstate, line->styleline->size);

		/* draw all the lines */
		for ( ; line!=&g->text->lines && ypos<info->ysiz ; line=line->next )
		{
			/* move to start of line */
			PFMoveTo(info->gstate, xpos, ypos+line->baseline);

			/* run through elements in line and draw all */
			for (el=line->elements ; el ; el=el->next)
				el->Draw(info->gstate,el);

			/* increment current position to go to next line */
			ypos+=line->height;
		}
	}
	return ERR_OK;
}

static Error move(PWObject object, CanvasInfo *info)
{
	Global *g;
	pt start,pos;
	char inlink=FALSE;
	TextLine *line;
	LinkArea *link;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	if (!g->text) return ERR_OK;

	if (!(line=g->line)) return ERR_OK;
	start=g->ypos;
	pos=info->ypos+info->yorg;
	while (line!=&g->text->lines && pos>start+line->height)
	{
		start+=line->height;
		line=line->next;
	}

	if (line!=&g->text->lines)
	{
		/* determine whether the pointer is on a link */
		pos=info->xpos+info->xorg-MARGIN;
		link=line->links;
		while (link && pos<link->xorg) link=link->next;
		if (link && pos<=link->xend) inlink=TRUE;
	}
	if (g->inlink!=inlink)
	{
		PWChange(object, PW_POINTER, inlink?SPRITE_HAND:NULL , NULL);
		g->inlink=inlink;
	}

	return ERR_OK;
}

static Error hit(PWObject object, CanvasInfo *info)
{
	Global *g;
	pt start,pos;
	char inlink=FALSE;
	TextLine *line;
	LinkArea *link;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	if (!g->text) return ERR_OK;

	line=g->line;
	start=g->ypos;
	pos=info->ypos+info->yorg;
	while (line!=&g->text->lines && pos>start+line->height)
	{
		start+=line->height;
		line=line->next;
	}

	if (line!=&g->text->lines)
	{
		/* determine whether the pointer is on a link */
		pos=info->xpos+info->xorg-MARGIN;
		link=line->links;
		while (link && pos<link->xorg) link=link->next;
		if (link && pos<=link->xend) inlink=TRUE;
	}
	if (inlink)
	{
		if ( STRSameCI(link->destination->filename,g->filename) ||
			 link->destination->filename[0]=='\0' )
		{
			ApplicPosition(g,link->destination->position);
		}
		else
		{
			char position[MAXLINKPOSITION]; /* copy as heap can be released! */
			STRCopy(link->destination->position,position);
			STRCopy(link->destination->filename,g->filename);
			FileDisplay(g,position);
		}
		
		if (g->text)
			PWChange(g->title,PW_INFOSTRING_TEXT,g->text->title,NULL);
		g->loaded=FALSE;
	}

	return ERR_OK;
}

static Error resetinlink(PWObject object, CanvasInfo *info)
{
	Global *g;
	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	/* the pointer itself is automatically restored by exitting the canvas */
	g->inlink=FALSE;
	return ERR_OK;
}

Error ApplicCreate(Global *g)
{
	return PWCreate(g->outline,&g->applic,PW_TYPE_APPLIC,
					PW_APPLIC_SCROLLBAR_X,
					PW_APPLIC_SCROLLBAR_Y,
					PW_APPLIC_CANVAS_LIST,
						PW_CANVAS_YSIZE, short2pt(30),
						PW_CANVAS_ACTION_SCALE, scale,
						PW_CANVAS_ACTION_REDRAW, draw,
						PW_CANVAS_ACTION_MOVE, move,
						PW_CANVAS_ACTION_HIT, hit,
						PW_CANVAS_ACTION_DO, hit,
						PW_CANVAS_ACTION_EXIT, resetinlink,
						NULL,
					NULL);
}

Error ApplicRedisplay(Global *g)
{
	return PWChange(g->applic,
					PW_APPLIC_CANVAS_LIST,
						PW_CANVAS_REDRAW,
						NULL,
					NULL);
}

Error ApplicPosition(Global *g, char *position)
{
	TextLine *line;
	pt pos=0;
	char found=FALSE;

	if (!g->text) return ERR_OK;

	PWChange(g->applic,
			PW_APPLIC_XSCROLL_LIST,
				PW_SCROLL_MINDIST, short2pt(20),
				PW_SCROLL_MAXDIST, -short2pt(20),
				PW_SCROLL_MAXIMUM, g->text->width+2*MARGIN,
				PW_SCROLL_SCROLL, ptmin,
				PW_SCROLL_MIN_KEYPRESSES, 193, 201,
				PW_SCROLL_MAX_KEYPRESSES, 197, 205,
				NULL,
			PW_APPLIC_YSCROLL_LIST,
				PW_SCROLL_MINDIST, short2pt(20),
				PW_SCROLL_MAXDIST, -short2pt(20),
				PW_SCROLL_MAXIMUM, g->text->height+2*MARGIN,
				PW_SCROLL_SCROLL, ptmin,
				PW_SCROLL_MIN_KEYPRESSES, 209, 217,
				PW_SCROLL_MAX_KEYPRESSES, 213, 221,
				NULL,
			NULL);

	if (!*position) return ERR_OK;	/* need a position to serach for */

	line=g->text->lines.next;

	while ( !found && (line!=&g->text->lines) )
	{
		Marker *marker=line->markers;
		while (marker && !STRSameCI(marker->name,position))
			marker=marker->next;
		if (marker)
			found=TRUE;
		else
		{
			pos+=line->height;
			line=line->next;
		}
	}
	if (found)
	{
		PWChange(g->applic,
					PW_APPLIC_YSCROLL_LIST,
						PW_SCROLL_SCROLL, pos+MARGIN,
						NULL,
					NULL);
	}
	return ERR_OK;
}
